C***********************************************************************
                PROGRAM TEST_SLIPS2
C***********************************************************************
C                                                                      *
C----------------------------------------------------------------------*
C                      DUBNA                                           *
C         JOINT INSTITUTE FOR NUCLEAR RESEARCH                         *
C        LABORATORY OF INFORMATION TECHNOLOGIES                        *
C                  P11-89-728, 1989                                    *
C                    PUZYNINA T.P.                                     *
C   SLIPS2 - PROGRAM FOR SOLUTION OF STURM-LIOUVILLE'S PROBLEM         *
C    FOR SYSTEM OF TWO SECOND-ORDER DIFFERENTIAL EQUATIONS             *
C                                                                      *
C----------------------------------------------------------------------*
C                        TEST:                                         *
C  1-TH EQUATION:  D2(HI1)/DX**2-(DEXP(-X)+1-2/X)*HI1+2*M*E*HI2=0      *
C  2-TH EQUATION:  D2(HI2)/DX**2-(DEXP(X)+4-4/X)*HI2+2*M*E*HI1=0       *
C             X ON INTERVAL (A,B).   A=0   B=10                        *
C  BOUNDARY CONDITIONS: HI1(0)=HI2(0)=0  B*D(HI1(B))/DX+(B-1)*HI1(B)=0 *
C                         B*D(HI2(B))/DX+(2*B-1)*HI2(B)=0              *
C                                                                      *
C  ANALYTICAL SOLUTION: LAMBDA=1 HI1(X)=X*DEXP(-X) HI2(X)=X*DEXP(-2*X) *
C----------------------------------------------------------------------*
C
COMMENTS:
C  HERE IS  LAMBDA=2.*M*E.   IF M=0.5 THEN E=LAMBDA
C
C  THE RICHARDSON'S EXTRAPOLATION IS HERE GIVEN
C
C-------------------------------------------------------------------

      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NMAX=401)
      COMMON /AMATR/ AK11(NMAX),AK12(NMAX),AK21(NMAX),AK22(NMAX),
     ,G11(NMAX),G12(NMAX),G21(NMAX),G22(NMAX),
     ,Q11(NMAX),Q12(NMAX),Q21(NMAX),Q22(NMAX)
      COMMON /BOUND/ D1(2),DD1(2),F1(2),DF1(2),D2(2),DD2(2),F2(2),DF2(2)
      COMMON /MOD/ MOD,NSOL
      DIMENSION V1(NMAX),V2(NMAX),W1(NMAX),W2(NMAX),R(NMAX)
      DIMENSION AV(NMAX),BV(NMAX),C1V(NMAX),C2V(NMAX)
      DIMENSION HI10(NMAX),HI20(NMAX),HI11(NMAX),HI21(NMAX)
C
C  NMAX IS MAXIMAL DIMENSION FOR ALL ARRAYS
C
      MOD=0

      IF(MOD.EQ.0)GO TO 5
      OPEN(1,FILE='SLIPS2_1.DAT')
      OPEN(2,FILE='SLIPS2_2.DAT')
    5 OPEN(3,FILE='SLIPS2_3.DAT')

      A=0.D0
      B=10.D0
      AM=0.5D0
      TAU0=0.1D0
      ITER=50
      EPS=1.D-5
      LST=3
      LDEL=1
      NHIT=5
      NR=101
      PRINT 99,NMAX,A,B,AM
      REWIND 3
CYCLE FOR H, H/2, H/4
      DO 10 NC=1,3
      IF(NR.GT.NMAX) PRINT 115,NR,NMAX
      NH=NR/10
      NR1=NR-1
      HR=(B-A)/NR1
      NSOL=0
      E0=0.5D0
      IF(NC.GT.1)E0=E
      PRINT 100,NR,HR,NH
      ALAMBD=2.D0*AM*E0
      R(1)=A
      R(NR)=B
      DO 1 I=2,NR1
      R(I)=A+(I-1)*HR
      AK11(I)=DEXP(-R(I))+1.D0-2.D0/R(I)
      AK12(I)=0.D0
      AK21(I)=0.D0
      AK22(I)=DEXP(R(I))+4.D0-4.D0/R(I)
      Q11(I)=0.D0
      Q12(I)=0.D0
      Q21(I)=0.D0
      Q22(I)=0.D0
      G11(I)=0.D0
      G12(I)=1.D0
      G21(I)=1.D0
      G22(I)=0.D0
   1  CONTINUE
      IF(NC.GT.1)GO TO 2
      PRINT 101,MOD,ITER,NHIT,EPS,LST,LDEL,NSOL,E0,ALAMBD,TAU0
      PRINT 102
      PRINT 103,(J,R(J),AK11(J),AK12(J),AK21(J),AK22(J),
     ,G11(J),G12(J),G21(J),G22(J),J=1,NR,NH)
   2  CALL D1F1(E0,A,AM)
      CALL D2F2(E0,B,AM)
      IF(NC.GT.1)GOTO 6
COMMENT:  FOR NC=1 INITIAL APPROXIMATION HI10(I)=HI20(I)
      CALL SINNZ(NR,R,HI10,0,0,0)
      CALL SINNZ(NR,R,HI20,0,0,0)
      GO TO 7
   6  CALL INITIAL(NR,HI11,HI10)
      CALL INITIAL(NR,HI21,HI20)
   7  CONTINUE
      CALL NORMS2(HR,NR,HI10,HI20)

      CALL SLIPS2(NR,R,AM,E0,E,TAU0,EPS,ITER,NHIT,LST,LDEL,NH,
     ,HI10,HI20,HI11,HI21,V1,V2,W1,W2,AV,BV,C1V,C2V)
      IF(MOD.EQ.2)CALL ORTS2(NSOL,NR,HR,HI11,HI21,V1,V2,W1,W2,1,2,1)
      CALL ANALYT(NR,R,HI10,HI20)
      CALL NORMS2(HR,NR,HI10,HI20)
      PRINT 104
      PRINT 105,(J,R(J),HI10(J),HI11(J),HI20(J),HI21(J),J=1,NR,NH)
      WRITE(3,106)E,NR,NH,(HI11(J),HI21(J),J=1,NR,NH)
      NR=2*NR-1
      PRINT 117
   10 CONTINUE
      REWIND 3
      NN=NR/(2*NH)+1
      READ(3,106)E1,NR1,NH1,(HI11(J),HI21(J),J=1,NN)
      READ(3,106)E2,NR2,NH2,(V1(J),V2(J),J=1,NN)
      READ(3,106)E3,NR3,NH3,(W1(J),W2(J),J=1,NN)
      OTNE=(E1-E2)/(E2-E3)
      PRINT 111,OTNE
      DO 3 I=2,NN
      AV(I)=(HI11(I)-V1(I))/(V1(I)-W1(I))
      BV(I)=(HI21(I)-V2(I))/(V2(I)-W2(I))
      R(I)=R(1+NH*(I-1))
    3 CONTINUE
      PRINT 112
      PRINT 113,(R(J),HI11(J),V1(J),W1(J),AV(J),J=2,NN)
      PRINT 114
      PRINT 113,(R(J),HI21(J),V2(J),W2(J),BV(J),J=2,NN)
      CALL EXTRAR(2,4,E1,E2,E3,E,A1,A2,A3)
      PRINT 107,E1,E2,E3
      EANAL=1.D0
      PRINT 108,E,EANAL
      SA=A1+A2+A3
      DO 4 J=1,NN
      HI11(J)=(A1*HI11(J)+A2*V1(J)+A3*W1(J))/SA
      HI21(J)=(A1*HI21(J)+A2*V2(J)+A3*W2(J))/SA
      HI10(J)=HI10(1+NH*(J-1))
      HI20(J)=HI20(1+NH*(J-1))
    4 CONTINUE
      PRINT 109
      PRINT 110,(R(J),HI10(J),HI11(J),HI20(J),HI21(J),J=1,NN)
      PRINT 116

   99 FORMAT(1X,'S L I P S 2:    T E S T  R U N  O U T P U T',//,
     ,1X,' NMAX=',I4,'   A=',F3.1,'   B=',F9.3,'   M=',F3.1)
  100 FORMAT(/,1X,' N=',I5,' H=',D9.3,' NH=',I3)
  101 FORMAT(/,1X,' MOD=',I1,' ITER=',I4,' NHIT=',I4,' EPS=',D9.3,
     ,' LTAU=',I1,' LDEL=',I1,' NSOL=',I2,/,1X,' EV0=',D9.3,
     ,' LAMBDA0=',D9.3,' TAU0=',F9.3)
  102 FORMAT(/,4X,'J',4X,'R',7X,'K11',7X,'K12',2X,'K21',6X,'K22',
     ,7X,'G11',2X,'G12',2X,'G21',2X,'G22')
  103 FORMAT(I5,F6.2,D14.6,2F5.1,D14.6,4F5.1)
  104 FORMAT(/,4X,'J',5X,'R',5X,'Y1 ANALYT',8X,'Y1',11X,'Y2 ANALYT',
     ,8X,'Y2')
  105 FORMAT(I5,F8.2,4D15.8)
  106 FORMAT(D24.16,2I12,(2D24.16))
  107 FORMAT(//,19X,'E(H)=',D16.9,/,17X,'E(H/2)=',D16.9,/
     ,17X,'E(H/4)=',D16.9)
  108 FORMAT(15X,'E EXTRAP=',D16.9,/,15X,'E ANALYT=',D16.9)
  109 FORMAT(/,5X,'R',6X,'Y1 ANALYT',7X,'Y1 EXTRAP',7X,'Y2 ANALYT',
     ,7X,'Y2 EXTRAP')
  110 FORMAT(F8.2,4D16.8)
  111 FORMAT(/,13X,'(E(H)-E(H/2))/(E(H/2)-E(H/4))=',F4.2)
  112 FORMAT(/,4X,'R',8X,'Y1(H)',9X,'Y1(H/2)',8X,'Y1(H/4)',
     ,5X,'SIGMA1')
  113 FORMAT(F8.2,3D15.8,F7.2)
  114 FORMAT(/,4X,'R',8X,'Y2(H)',9X,'Y2(H/2)',8X,'Y2(H/4)',
     ,5X,'SIGMA2')
  115 FORMAT(/,1X,'ATTENSION!  NR=',I6,' .GT. NMAX=',I6,/)
  116 FORMAT(/,1X,'****************END***TEST***SLIPS2****************')
  117 FORMAT(/,1X,'***************************************************')
      END
C
C***********************************************************************
C
      SUBROUTINE SINNZ(N,X,Y,NZ,LYA,LYB)
C
C  CALCULATION OF INITIAL APPROXIMATION IN FORM
C  Y(X)=SIN(PI*(NZ+1)*((X-A)/(B-A))
C  WITH Y(A)=0. FOR LYA=0  AND DY(A)/DX=0. FOR LYA.NE.0
C  WITH Y(B)=0. FOR LYB=0  AND DY(B)/DX=0. FOR LYB.NE.0
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(N),Y(N)
      NZP=NZ+1
      IL=N/(2*NZP)
      IR=N-IL
      C=3.14159264D0*NZP/(X(N)-X(1))
      DO 1 I=1,N
   1  Y(I)=DSIN(C*(X(I)-X(1)))
      IF(LYA.EQ.0)GO TO 3
      DO 2 J=1,IL
   2  Y(J)=1.D0
   3  IF(LYB.EQ.0)RETURN
      DO 4 K=IR,N
   4  Y(K)=Y(IR)
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE INITIAL(N,Y0,Y)
C
C  CALCULATION OF NEXT INITIAL APPROXIMATION Y(X)
C  FROM FUNCTION Y0 WITH HELP OF LINEAR INTERPOLATION
C

      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y0(N),Y(N)

      N2=(N-1)/2
      Y(1)=Y0(1)

      DO I=1,N2
      Y(2*I+1)=Y0(I+1)
      Y(2*I)=0.5D0*( Y(2*I+1)+Y(2*I-1) )
      ENDDO

      RETURN
      END
C
C***********************************************************************
C

      SUBROUTINE ANALYT(NR,R,Y1,Y2)
C
C  CALCULATION OF ANALYTICAL SOLUTIONS Y1 AND Y2 FOR TEST
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION R(NR),Y1(NR),Y2(NR)
      DO 1 I=1,NR
      EX=DEXP(-R(I))
      Y1(I)=R(I)*EX
   1  Y2(I)=Y1(I)*EX
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE EXTRAR(N1,N2,E1,E2,E3,E,A1,A2,A3)
C
C  CALCULATION OF COEFFICIENTS A1,A2,A3 FOR RICHARDSON'S EXTRAPOLATION
C
      IMPLICIT REAL*8(A-H,O-Z)
      A3=1.D0
      P2N1=2.D0**N1
      P2N2=2.D0**N2
      A1=A3/(P2N1*P2N2)
      A2=-A1*(P2N1+P2N2)
      E=(A1*E1+A2*E2+A3*E3)/(A1+A2+A3)
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE SLIPS2(NR,R,AM,EV0,EV,T0,EPS,ITER,NHIT,LST,LDEL,LR,
     ,HI10,HI20,HI1,HI2,AV,BV,AW,BW,C1V,C2V,C1W,C2W)
C
C  SUBROUTINE FOR REALIZATION OF DISCRETE ANALOG OF NEWTON'S METHOD
C  FOR SOLUTION OF STURM-LIOUVILLE'S PROBLEM
C
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/MOD/MOD,NSOL
      DIMENSION HI10(NR),HI20(NR),HI1(NR),HI2(NR),R(NR)
      DIMENSION AV(NR),BV(NR),C1V(NR),C2V(NR)
      DIMENSION AW(NR),BW(NR),C1W(NR),C2W(NR)
      A=R(1)
      B=R(NR)
      HR=R(2)-R(1)
      M=1
      TK=T0
      TK1=T0
      EV=EV0
      EVF=EV0
      MODF=MOD
      LRET=0
      DO 1 I=1,NR
      HI1(I)=HI10(I)
    1 HI2(I)=HI20(I)
      CALL NORMS2(HR,NR,HI1,HI2)
      CALL DELTA2(EV,AM,NR,R,HI1,HI2,DELK,RDEL,NEQ,DEL2)
      ALAM=2.D0*AM*EV
      PRINT 99,EV,ALAM
      PRINT 100,DELK,RDEL,NEQ,DEL2,MOD,LST,LDEL
      IF(LST.EQ.4.OR.LDEL.EQ.2)DELK=DEL2
      DELK1=DELK
      DELK2=DELK
      DO 20 K=1,ITER
      EVFIX=EV
      IF(MOD.EQ.2)EVFIX=EVF
      IF(DELK.GT.EPS) GO TO 7
C* RETURN TO INPUT MOD
      MOD=MODF
      PRINT 101,EPS
      LRET=1
    2 K1=K-1
      IF(K1.EQ.0)GO TO 6
      IF(LST.EQ.4.OR.LDEL.EQ.2)GO TO 4
    3 PRINT 102,K1,DELK,RDEL,NEQ
      GO TO 5
    4 PRINT 109,K1,DELK
    5 PRINT 103,EV,ALAM,AMJU,TK,MOD
    6 PRINT 104
      PRINT 105,(J,R(J),HI1(J),HI2(J),J=1,NR,LR)
      IF(LRET.EQ.1)RETURN
      M=M+NHIT
    7 IF(K.EQ.M)GO TO 2
C* RETURN FROM MOD=2 TO MOD=0
      IF(MOD.EQ.0)GO TO 8
      OTN=DELK2/DELK
      COTN=1.5D0
      ITMIN=10
      CONST=1.D+3
      IF(OTN.GT.COTN.OR.K.LT.ITMIN)GO TO 8
      MOD=0
      EV=EV0
      PRINT 106,EV
    8 CALL PROGS2(A,B,NR,HI1,HI2,AM,EV,EVFIX,C,
     ,AV,BV,AW,BW,C1V,C2V,C1W,C2W)
      IF(C.EQ.0)GO TO 10
    9 PRINT 107
      RETURN
   10 IF(MOD.EQ.2)GO TO 12
      DO 11 I=1,NR
      AV(I)=-HI1(I)
   11 AW(I)=-HI2(I)
   12 CALL SIMPS(HR,NR,HI1,HI1,S1)
      CALL SIMPS(HR,NR,HI2,HI2,S2)
      CALL SIMPS(HR,NR,HI1,AV,S3)
      CALL SIMPS(HR,NR,HI2,AW,S4)
      CALL SIMPS(HR,NR,HI1,BV,S5)
      CALL SIMPS(HR,NR,HI2,BW,S6)
      AMJU=0.5D0*(1.D0-S1-S2-2.D0*S3-2.D0*S4)/(S5+S6)
      DO 13 I=1,NR
      AV(I)=AV(I)+AMJU*BV(I)
   13 AW(I)=AW(I)+AMJU*BW(I)
      CALL TAUKA2(LST,DELK1,DELK,T0,TK1,TK,NR,R,AM,EV,AMJU,HI1,HI2,
     ,AV,AW,BV,BW)
      DO 14 I=1,NR
      HI1(I)=HI1(I)+TK*AV(I)
   14 HI2(I)=HI2(I)+TK*AW(I)
      CALL BOUND2(NR,HI1,HI2)
      EV=EV+TK*AMJU
      ALAM=2.D0*AM*EV
      DELK2=DELK1
      DELK1=DELK
      TK1=TK
      IF(MODF.EQ.2) CALL ORTS2(NSOL,NR,HR,HI1,HI2,AV,BV,AW,BW,1,2,0)
      CALL NORMS2(HR,NR,HI1,HI2)
      CALL DELTA2(EV,AM,NR,R,HI1,HI2,DELK,RDEL,NEQ,DEL2)
      IF(LST.EQ.4.OR.LDEL.EQ.2)DELK=DEL2
   20 CONTINUE
      PRINT 108,ITER
      IF(LST.EQ.4.OR.LDEL.EQ.2)GO TO 15
      PRINT 102,ITER,DELK,RDEL,NEQ
      GO TO 16
   15 PRINT 109,ITER,DELK
   16 PRINT 103,EV,ALAM,AMJU,TK,MOD
      PRINT 104
      PRINT 105,(J,R(J),HI1(J),HI2(J),J=1,NR,LR)
      RETURN
   99 FORMAT(/,1X,'EV0=',D13.6,' LAMBDA0=',D13.6)
  100 FORMAT(/,1X,'DELTA10=',D9.2,' AT R=',D13.6,
     ,' FOR EQUATION ',I1,/ ,1X,'MEAN SQUARE DELTA20=',D13.6,
     ,/,1X,'  MOD=',I1,' LST=',I1,' LDEL=',I1)
  101 FORMAT(/,1X,'SOLUTION OF SYSTEM WITH GIVEN PRECISION EPS=',D10.3)
  102 FORMAT(/,1X,'ITER=',I3,' DELTA=',D9.2,' AT R=',D13.6,
     ,' FOR EQUATION ',I1)
  103 FORMAT(1X,'EV=',D13.6,' LAMBDA=',D13.6,/,3X,
     ,'MJU=',D13.6,' TAUKA=',D13.6,' MOD=',I1)
  104 FORMAT(/,5X,'J',7X,'R',11X,'HI1',14X,'HI2')
  105 FORMAT(2X,I4,F10.4,2D18.9)
  106 FORMAT(1X,'GO FROM MOD=2 TO M0D=0, EV=',D13.6)
  107 FORMAT(/,' RETURN FROM SLIPS2: PROGONKA INCORRECT')
  108 FORMAT(/,1X,'RETURN FOR NUMBER OF ITERATIONS=',I4)
  109 FORMAT(/,1X,'ITER=',I3,' DELTA=',D13.6)
      END
C
C***********************************************************************
C
      SUBROUTINE D1F1(EV,A,AM)
C
C  CALCULATION OF COEFFICIENTS D1,F1 AND DERIVATIVES DD1,DF1 IN
C  BOUNDARY CONDITIONS AT X=A. EV IS EIGENVALUE LAMBDA FOR M=0.5
C  THERE IS LAMBDA=2.*M*EV USUALLY
C
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/BOUND/D1(2),DD1(2),F1(2),DF1(2),D2(2),DD2(2),F2(2),DF2(2)
      D1(1)=0.D0
      D1(2)=0.D0
      F1(1)=1.D0
      F1(2)=1.D0
      DD1(1)=0.D0
      DD1(2)=0.D0
      DF1(1)=0.D0
      DF1(2)=0.D0
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE D2F2(EV,B,AM)
C
C  CALCULATION OF COEFFICIENTS D2,F2 AND DERIVATIVES DD2,DF2 IN
C  BOUNDARY CONDITIONS AT X=B. EV IS EIGENVALUE LAMBDA FOR M=0.5
C  THERE IS LAMBDA=2.*M*EV USUALLY
C
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /BOUND/D1(2),DD1(2),F1(2),DF1(2),D2(2),DD2(2),F2(2),DF2(2)
      D2(1)=B
      D2(2)=B
      F2(1)=B-1.D0
      F2(2)=2.D0*B-1.D0
      DD2(1)=0.D0
      DD2(2)=0.D0
      DF2(1)=0.D0
      DF2(2)=0.D0
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE DELTA2(E0,AM,NR,R,HI1,HI2,DEL,RDEL,NEQ,DEL2)
C
C  CALCULATION RESIDUAL DELTA FOR DISCRETE FORM PROBLEM
C
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NMAX=401)
      COMMON /AMATR/ AK11(NMAX),AK12(NMAX),AK21(NMAX),AK22(NMAX),
     ,G11(NMAX),G12(NMAX),G21(NMAX),G22(NMAX),
     ,Q11(NMAX),Q12(NMAX),Q21(NMAX),Q22(NMAX)
      DIMENSION R(NR),HI1(NR),HI2(NR)
      NR1=NR-1
      HR=R(2)-R(1)
      DEL=0.D0
      NEQ=1
      DEL2=0.D0
      DTRAP=0.D0
      DO 1 I=2,NR1
      D1=ABS((HI1(I+1)-2.D0*HI1(I)+HI1(I-1))/HR**2-Q11(I)*
     *(HI1(I+1)-HI1(I-1))/HR-(AK11(I)-2.D0*AM*E0*G11(I))*HI1(I)-
     -(AK12(I)-2.D0*AM*E0*G12(I))*HI2(I)-Q12(I)*(HI2(I+1)-HI2(I-1))/HR)
      DEL2=DEL2+2.D0*D1*D1
      DTRAP=DTRAP+D1*D1
      IF((I/2).EQ.((I+1)/2))DEL2=DEL2+2.D0*D1*D1
      IF(D1.LT.DEL) GO TO 1
      DEL=D1
      RDEL=R(I)
   1  CONTINUE
      DO 2 I=2,NR1
      D2=ABS((HI2(I+1)-2.D0*HI2(I)+HI2(I-1))/HR**2-Q22(I)*
     *(HI2(I+1)-HI2(I-1))/HR-(AK22(I)-2.D0*AM*E0*G22(I))*HI2(I)-
     -(AK21(I)-2.D0*AM*E0*G21(I))*
     *HI1(I)-Q21(I)*(HI1(I+1)-HI1(I-1))/HR)
      DEL2=DEL2+2.D0*D2*D2
      DTRAP=DTRAP+D2*D2
      IF((I/2).EQ.((I+1)/2))DEL2=DEL2+2.D0*D2*D2
      IF(D2.LT.DEL) GO TO 2
      DEL=D2
      RDEL=R(I)
      NEQ=2
    2 CONTINUE
      DEL2=3.D0*HR*DEL2
      DTRAP=HR*DTRAP
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE PROGS2(A,B,M,HI10,HI20,AM,E,EF,C,
     ,AV,BV,AW,BW,C1V,C2V,C1W,C2W)
C
C  PROGONKA (SWEEP)  METHOD FOR SOLUTION OF EQUATIONS WITH GIVEN
C  BOUNDARY CONDITIONS
C
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NMAX=401)
      COMMON /AMATR/ AK11(NMAX),AK12(NMAX),AK21(NMAX),AK22(NMAX),
     ,G11(NMAX),G12(NMAX),G21(NMAX),G22(NMAX),
     ,Q11(NMAX),Q12(NMAX),Q21(NMAX),Q22(NMAX)
      COMMON /BOUND/ D1(2),DD1(2),F1(2),DF1(2),D2(2),DD2(2),F2(2),DF2(2)
      DIMENSION HI10(M),HI20(M)
      DIMENSION AV(M),BV(M),C1V(M),C2V(M)
      DIMENSION AW(M),BW(M),C1W(M),C2W(M)
      DIMENSION CC(6,8),INDEX(6)
      AV(1)=0.D0
      BV(1)=0.D0
      AW(1)=0.D0
      BW(1)=0.D0
      C1V(1)=0.D0
      C2V(1)=0.D0
      C1W(1)=0.D0
      C2W(1)=0.D0
      C=0.D0
      M1=M-1
      M2=M-2
      H=(B-A)/M1
      DO 1 I=2,M1
      AN1=1.D0-H*Q11(I)
      AN0=(2.D0*AM*EF*G11(I)-AK11(I))*H**2-2.D0
      ANM1=1.D0+H*Q11(I)
      AM1=-H*Q12(I)
      AM0=-(AK12(I)-2.D0*AM*EF*G12(I))*H**2
      AMM1=H*Q12(I)
      AL1=-H*Q21(I)
      BW(1)=0.D0
      AL0=-(AK21(I)-2.D0*AM*EF*G21(I))*H**2
      ALM1=H*Q21(I)
      P1=1.D0-H*Q22(I)
      P0=(2.D0*AM*EF*G22(I)-AK22(I))*H**2-2.D0
      PM1=1.D0+H*Q22(I)
      R1=HI10(I+1)*(H*Q11(I)-1.D0)+HI10(I)*(2.D0+H**2*(AK11(I)-
     -2.D0*AM*E*G11(I)))-HI10(I-1)*(1.D0+H*Q11(I))+HI20(I+1)*Q12(I)*H+
     +HI20(I)*(AK12(I)-2.D0*AM*E*G12(I))*H**2-HI20(I-1)*Q12(I)*H
      S1=HI20(I+1)*(H*Q22(I)-1.D0)+HI20(I)*(2.D0+H**2*(AK22(I)-
     -2.D0*AM*E*G22(I)))-HI20(I-1)*(1.D0+H*Q22(I))+HI10(I+1)*Q21(I)*H+
     +HI10(I)*(AK21(I)-2.D0*AM*E*G21(I))*H**2-HI10(I-1)*Q21(I)*H
      R2=-2.D0*AM*(HI10(I)*G11(I)+HI20(I)*G12(I))*H**2
      S2=-2.D0*AM*(HI20(I)*G22(I)+HI10(I)*G21(I))*H**2
      ALFA11=AN0+ANM1*AV(I-1)+AMM1*AW(I-1)
      ALFA12=AM0+ANM1*BV(I-1)+AMM1*BW(I-1)
      ALFA21=AL0+ALM1*AV(I-1)+PM1*AW(I-1)
      ALFA22=P0+ALM1*BV(I-1)+PM1*BW(I-1)
      DD=ALFA11*ALFA22-ALFA12*ALFA21
      IF(ABS(DD)-1.D-25)2,2,3
    2 PRINT 100
      C=1.D0
      RETURN
    3 BETA11=-AN1
      BETA12=-AM1
      BETA21=-AL1
      BETA22=-P1
      GAMMA1=R1-ANM1*C1V(I-1)-AMM1*C1W(I-1)
      GAMMA2=S1-ALM1*C1V(I-1)-PM1*C1W(I-1)
      GAMMA3=R2-ANM1*C2V(I-1)-AMM1*C2W(I-1)
      GAMMA4=S2-ALM1*C2V(I-1)-PM1*C2W(I-1)
      AV(I)=(ALFA22*BETA11-ALFA12*BETA21)/DD
      BV(I)=(ALFA22*BETA12-ALFA12*BETA22)/DD
      C1V(I)=(ALFA22*GAMMA1-ALFA12*GAMMA2)/DD
      C2V(I)=(ALFA22*GAMMA3-ALFA12*GAMMA4)/DD
      AW(I)=(ALFA11*BETA21-ALFA21*BETA11)/DD
      BW(I)=(ALFA11*BETA22-ALFA21*BETA12)/DD
      C1W(I)=(ALFA11*GAMMA2-ALFA21*GAMMA1)/DD
      C2W(I)=(ALFA11*GAMMA4-ALFA21*GAMMA3)/DD
    1 CONTINUE
   16 NS=4
      DO 15 I=1,6
      DO 15 J=1,8
   15 CC(I,J)=0.D0
      DHI1=HI10(M2)-4.D0*HI10(M1)+3.D0*HI10(M)
      DHI2=HI20(M2)-4.D0*HI20(M1)+3.D0*HI20(M)
      CALL D1F1(EF,A,AM)
      CALL D2F2(EF,B,AM)
      CC(1,1)=D2(1)
      CC(1,2)=-4.D0*D2(1)
      CC(1,3)=3.D0*D2(1)+2.D0*H*F2(1)
      CC(1,8)=-DD2(1)*DHI1-2.D0*H*DF2(1)*HI10(M)
      CC(2,4)=D2(2)
      CC(2,5)=-4.D0*D2(2)
      CC(2,6)=3.D0*D2(2)+2.D0*H*F2(2)
      CC(2,8)=-DD2(2)*DHI2-2.D0*H*DF2(2)*HI20(M)
      CC(3,1)=1.D0
      CC(3,2)=-AV(M2)
      CC(3,5)=-BV(M2)
      CC(3,7)=C1V(M2)
      CC(3,8)=C2V(M2)
      CC(4,4)=1.D0
      CC(4,2)=-AW(M2)
      CC(4,5)=-BW(M2)
      CC(4,7)=C1W(M2)
      CC(4,8)=C2W(M2)
      CC(5,2)=1.D0
      CC(5,3)=-AV(M1)
      CC(5,6)=-BV(M1)
      CC(5,7)=C1V(M1)
      CC(5,8)=C2V(M1)
      CC(6,5)=1.D0
      CC(6,3)=-AW(M1)
      CC(6,6)=-BW(M1)
      CC(6,7)=C1W(M1)
      CC(6,8)=C2W(M1)
      CALL D1F1(E,A,AM)
      CALL D2F2(E,B,AM)
      CC(1,7)=-D2(1)*DHI1-2.D0*H*F2(1)*HI10(M)
      CC(2,7)=-D2(2)*DHI2-2.D0*H*F2(2)*HI20(M)
C     PRINT 101,((CC(I,J),J=1,8),I=1,6)
      CALL LINEQ1(CC,6,6,8,2,INDEX,NERROR,DET2)
      IF(NERROR)12,14,13
   12 PRINT 102
      C=1.D0
      RETURN
   13 PRINT 103
      C=1.D0
      RETURN
   14 CONTINUE
      AV(M-2)=CC(1,1)
      AV(M-1)=CC(2,1)
      AV(M)=CC(3,1)
      AW(M-2)=CC(4,1)
      AW(M-1)=CC(5,1)
      AW(M)=CC(6,1)
      BV(M-2)=CC(1,2)
      BV(M-1)=CC(2,2)
      BV(M)=CC(3,2)
      BW(M-2)=CC(4,2)
      BW(M-1)=CC(5,2)
      BW(M)=CC(6,2)
      DO 5 J=4,M
      I=M-J+1
      V1=AV(I)*AV(I+1)+BV(I)*AW(I+1)+C1V(I)
      W1=AW(I)*AV(I+1)+BW(I)*AW(I+1)+C1W(I)
      V2=AV(I)*BV(I+1)+BV(I)*BW(I+1)+C2V(I)
      W2=AW(I)*BV(I+1)+BW(I)*BW(I+1)+C2W(I)
      AV(I)=V1
      AW(I)=W1
      BV(I)=V2
    5 BW(I)=W2
      RETURN
  100 FORMAT(/,5X,'PROGON.  DD=0.',/)
  101 FORMAT(8D15.5)
  102 FORMAT(/,10X,'ERROR  IN DATA  OR  IN  CALL PROGON',/)
  103 FORMAT(1X,'SYSTEM BOUNDARY CONDITIONS ARE LINEAR DEPENDED')
      END
C
C***********************************************************************
C
      SUBROUTINE TAUKA2(LST,DELKM1,DELK,T0,TKM1,TK,NR,R,AM,EV,AMJU,
     ,Y1,Y2,AV,AW,Y11,Y21)
C
C  CALCULATION OF PARAMETER TK FOR REGULARIZATION ITERATION PROCESS
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y1(NR),Y2(NR),AV(NR),AW(NR),Y11(NR),
     ,Y21(NR),R(NR),D(10)
      HR=R(2)-R(1)
      IF(LST.EQ.1) GO TO 10
      IF(LST.EQ.3) GO TO 8
      IF(LST.EQ.4) GO TO 11
      IF(LST.EQ.5) GO TO 12
    1 IF(DELK-DELKM1) 2,3,3
    2 TK=TKM1*2.D0
      IF(TK-1.) 5,5,4
    4 TK=1.D0
    5 RETURN
    3 TK=TKM1/2.D0
      IF(TK-T0) 6,7,7
    6 TK=T0
    7 RETURN
    8 TK=TKM1*DELKM1/DELK
      IF(TK-T0) 6,7,9
    9 IF(TK-1.) 7,7,4
   10 TK=T0
      RETURN
   11 DO 16 I=1,NR
      Y11(I)=Y1(I)+AV(I)
   16 Y21(I)=Y2(I)+AW(I)
      CALL DELTA2(EV,AM,NR,R,Y11,Y21,DEL,RDEL,NEQ,DEL2)
      TK=DELKM1/(DELKM1+DEL2)
      RETURN
   12 NTAU=10
      HTAU=(1.D0-T0)/(NTAU-1)
      DO 14 I=1,NTAU
      TK=T0+HTAU*(I-1)
      EV1=EV+TK*AMJU
      DO 13 J=1,NR
      Y11(J)=Y1(J)+TK*AV(J)
   13 Y21(J)=Y2(J)+TK*AW(J)
      CALL DELTA2(EV1,AM,NR,R,Y11,Y21,D(I),RDEL,NEQ,DEL2)
   14 CONTINUE
      DELK=D(1)
      DO 15 I=1,NTAU
      IF(DELK.LT.D(I)) GO TO 15
      ITAU=I
      DELK=D(I)
   15 CONTINUE
      TK=T0+(ITAU-1)*HTAU
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE BOUND2(NR,Y1,Y2)
C
C  SEND 0. ON FUNCTIONS WITH ZEROS BOUNDARY CONDITIONS
C
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/BOUND/D1(2),DD1(2),F1(2),DF1(2),D2(2),DD2(2),F2(2),DF2(2)
      DIMENSION Y1(NR),Y2(NR)
      IF(D1(1).EQ.0.)Y1(1)=0.D0
      IF(D1(2).EQ.0.)Y2(1)=0.D0
      IF(D2(1).EQ.0.)Y1(NR)=0.D0
      IF(D2(2).EQ.0.)Y2(NR)=0.D0
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE NORMS2(H,N,Y1,Y2)
C
C  THE NORMALIZATION OF FUNCTIONS Y1,Y2
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y1(N),Y2(N)
      CALL SIMPS(H,N,Y1,Y1,RY1)
      CALL SIMPS(H,N,Y2,Y2,RY2)
      ANORM=1.D0/DSQRT(RY1+RY2)
      DO 1 I=1,N
      Y1(I)=Y1(I)*ANORM
      Y2(I)=Y2(I)*ANORM
    1 CONTINUE
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE ORTS2(K,N,H,Y1,Y2,V1,V2,W1,W2,NT1,NT2,LW)
C
C ORTOGONALIZATION OF FUNCTIONS Y1,Y2 FOR ALL PREVIOUSLY EIGEN FUNCTIONS
C NT1,NT2 ARE TWO TAPES OR FILES ON DISK, WERE FUNCTIONS ARE SAVING
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y1(N),Y2(N),V1(N),V2(N),W1(N),W2(N)
      REWIND NT1
      REWIND NT2
      IF(K.EQ.0) GO TO 6
      DO 1 I=1,N
      V1(I)=Y1(I)
    1 V2(I)=Y2(I)
      DO 3 J=1,K
      READ(NT1,30)W1
      READ(NT2,30)W2
      CALL SIMPS(H,N,Y1,W1,S1)
      CALL SIMPS(H,N,Y2,W2,S2)
      S=S1+S2
      DO 2 L=1,N
      V1(L)=V1(L)-S*W1(L)
    2 V2(L)=V2(L)-S*W2(L)
    3 CONTINUE
      DO 4 M=1,N
      Y1(M)=V1(M)
    4 Y2(M)=V2(M)
    6 CALL NORMS2(H,N,Y1,Y2)
      IF(K.NE.0.AND.LW.EQ.0)RETURN
      WRITE(NT1,30)Y1
      WRITE(NT2,30)Y2
      K1=K+1
C      PRINT 20,K1
      RETURN
   20 FORMAT(1X,I5,' SOLUTIONS WERE WRITTEN')
   30 FORMAT(1X,D24.16)
      END
C
C***********************************************************************
C
      SUBROUTINE SIMPS(H,N,Y,V,S)
C
C  SIMPSON'S RULE INTEGRATION FOR FUNCTION Y*V. S IS INTEGRAL
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y(N),V(N)
      N12=(N-1)/2
      S1=Y(1)*V(1)-Y(N)*V(N)
      S=-Y(N)*V(N)
      DO 1 J=1,N12
      I1=2*J
      I=(N12-J+1)*2
      S1=S1+4.D0*Y(I1)*V(I1)+2.D0*Y(I1+1)*V(I1+1)
      S=S+4.D0*Y(I)*V(I)+2.D0*Y(I+1)*V(I+1)
   1  CONTINUE
      S=(S+Y(1)*V(1))*H/3.D0
      S1=S1*H/3.D0
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE LINEQ1 (A,DIM1,N1,DIM2,N2,INDEX,NERROR,DETERM)
C
C  LIBRARY SUBROUTINE FOR CALCULATION SYSTEM LINEAR ALGEBRAIC EQUATIONS
C
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER DIM1,DIM,DIM2,PIVCOL,PIV1,TOPX,ENDX,TOPCOL,ENDCOL,EMAT
      DIMENSION A(*),INDEX(*)
      DIM=DIM1
      DETER=1.0D0
      N=N1
      EMAT=N+N2
      NMIN1=N-1
      PIVCOL=-DIM
C     MAIN LOOP TO CREATE TRIANGULAR
      DO 10 MAIN=1,NMIN1
      PIVOT=0.D0
      PIVCOL=PIVCOL+DIM+1
      PIV1=PIVCOL+N-MAIN
C     SEARCH PIVOT
      DO 2 I1=PIVCOL,PIV1
      IF(ABS(A(I1))-ABS(PIVOT)) 2,2,1
    1 PIVOT=A(I1)
      LPIV=I1
    2 CONTINUE
C     IS PIVOT DIFFERENT FROM ZERO
      IF(PIVOT) 3,15,3
C     IS IT NECESSARY TO BRING PIVOT TO DIAGONAL
    3 IF(LPIV-PIVCOL) 4,6,4
    4 DETER=-DETER
      LPIV=LPIV-DIM
      I1=PIVCOL-DIM
      DO 5 I2=MAIN,EMAT
      LPIV=LPIV+DIM
      I1=I1+DIM
      SWAP=A(I1)
      A(I1)=A(LPIV)
    5 A(LPIV)=SWAP
    6 DETER=DETER*PIVOT
      PIVOT=1.D0/PIVOT
C     MODIFY PIVOT COLUMN
      I1=PIVCOL+1
      DO 7 I2=I1,PIV1
    7 A(I2)=A(I2)*PIVOT
C     CONVERT THE SUBMATRIX AND RIGHT SIDES
      I3=PIVCOL
      IROW=MAIN+1
      DO 9 I1=IROW,N
      I3=I3+1
      I4=PIVCOL
      I5=I3
      DO 8 I2=IROW,EMAT
      I4=I4+DIM
      I5=I5+DIM
    8 A(I5)=A(I5)-A(I4)*A(I3)
    9 CONTINUE
   10 CONTINUE
      DETERM=DETER
      LIM=I5-DIM*N2
       IF(A(LIM)) 16,15,16
  16  DETERM=DETERM*A(LIM)
      NERROR=0
C     COMPUTE THE SOLUTIONS
      NO=N+1
      TOPX=NMIN1*DIM+1
      DO 13 I=NO,EMAT
      TOPX=TOPX+DIM
      ENDX=TOPX+N
      TOPCOL=N*DIM+1
      ENDCOL=TOPCOL+NMIN1
      DO 12 I1=1,NMIN1
      ENDX=ENDX-1
      TOPCOL=TOPCOL-DIM
      ENDCOL=ENDCOL-DIM-1
      A(ENDX)=A(ENDX)/A(ENDCOL+1)
      SWAP=A(ENDX)
      I3=TOPX-1
      DO 11 I2=TOPCOL,ENDCOL
      I3=I3+1
   11 A(I3)=A(I3)-A(I2)*SWAP
   12 CONTINUE
      A(TOPX)=A(TOPX)/A(1)
   13 CONTINUE
C     LEFTADJUST THE SOLUTIONS
      I=-DIM
      TOPX=NMIN1*DIM+1
      ENDX=TOPX+NMIN1
      DO 14 I1=NO,EMAT
      TOPX=TOPX+DIM
      ENDX=ENDX+DIM
      I=I+DIM
      I3=I
      DO 14 I2=TOPX,ENDX
      I3=I3+1
   14 A(I3)=A(I2)
      RETURN
C     ERROR EXIT
   15 NERROR=-1
      DETERM=DETER
      PRINT 100,MAIN,MAIN
      RETURN
  100 FORMAT(' LINEQ1 ..... THE ',I10,'. COLUMN OF THE MATRIX CONTAINS',
     ,' ONLY ZEROS AT THE  ',I10,'. ELIMINATIONSTEP.')
      END
C
C*******END*******TEST*******SLIPS2*************************************
